% drawShapes(shapes, posLight, pos, orient, res, Nx, Ny)
%
%

function varargout = drawShapes(shapes, posLight, pos, orient, res, Nx, Ny)
  
  % parfor config
  global PARFOR_CONFIG
 
  PARFOR_CONFIG.minBatchSize = 8;
  PARFOR_CONFIG.enableWaitBar = true;
  PARFOR_CONFIG.nameWaitBar = 'Drawing Shapes';
  
  % shape info
  N = numel(shapes);
  isInside = false(N, 1);
  for n = 1:N
    isInside(n) = shapes{n}.isInside(pos);
  end
  
  % init lighting and view orientation vectors
  [mOrientInv, mOrient] = Vec3.alignOpZ(orient);
  thetaX = ((1:Nx)' - (Nx / 2)) * res * pi / (180 * Nx);
  thetaY = ((1:Ny)' - (Ny / 2)) * res * pi / (180 * Nx);

  sinCosX = [sin(thetaX), cos(thetaX)];
  sinCosY = [sin(thetaY), cos(thetaY)];
  
  posLight = Vec3.normalize(posLight);
  
  vViewX = Vec3.rotateX(Vec3.zAxis, sinCosX);

  % setup partch data by column
  patchCol.x = zeros(4, Ny);
  patchCol.y = zeros(4, Ny);
  patchCol.c = zeros(3, Ny);
  patchData = patchCol(ones(Nx, 1), 1);
  
  parfor nnx = 1:Nx
    vViewXY = Vec3.rotateY(vViewX(:, nnx), sinCosY);
    orientXY = mOrient * vViewXY;
    
    % some Ny sized stuff
    tHit = zeros(Ny, N);
    posHit = zeros(3, Ny);
    vNormHit = zeros(3, Ny);
    cSurfHit = zeros(3, Ny);

    dirLight = zeros(3, Ny);
    dotLight = zeros(Ny, 1);
    isDirectLight = false(Ny, 1);
    
    % find time to next hit
    for n = 1:N
      if isInside(n)
        tHit(:, n) = shapes{n}.hitFromInside(pos, orientXY);
      else
        tHit(:, n) = shapes{n}.hitFromOutside(pos, orientXY);
      end
    end
    [tMin, nMin] = min(tHit, [], 2);
    
    % find normal vector at next hit
    nIsHit = tMin < Inf;
    for n = 1:N
      nHitThis = nIsHit & (nMin == n);
      if any(nHitThis)
        % hit position
        posHit(:, nHitThis) = Vec3.sum(pos, ...
          Vec3.prod(tMin(nHitThis), orientXY(:, nHitThis)));
        
        % surface normal
      if isInside(n)
        vNormHit(:, nHitThis) = -shapes{n}.surfaceNormal(posHit(:, nHitThis));
      else
        vNormHit(:, nHitThis) = shapes{n}.surfaceNormal(posHit(:, nHitThis));
      end
        
        % surface color
        cSurfHit(:, nHitThis) = shapes{n}.surfaceColor(posHit(:, nHitThis));
      end
    end
    
    % determine if each hit position can see the light
%     posHit
%     vNormHit
    dirLight(:, nIsHit) = Vec3.diff(posLight, posHit(:, nIsHit));
    dotLight(nIsHit) = Vec3.dot(vNormHit(:, nIsHit), dirLight(:, nIsHit));
    
    nCheckLight = dotLight > 0;
    isDirectLight(nCheckLight) = true;
    if any(nCheckLight)
      for n = 1:N
        if ~isInside(n)
          nCheckThis = nCheckLight;
          %nCheckThis = nCheckLight & (nMin ~= n);
          
          % compute flight time to light
          tLight = shapes{n}.hitFromOutside( ...
            posHit(:, nCheckThis), dirLight(:, nCheckThis));
          
          % did the ray get back to the light?
          isDirectLight(nCheckThis) = ...
            isDirectLight(nCheckThis) & (tLight > 1);
        end
      end
    end
    %[nnx, sum(isDirectLight)]
    
    % look for patches to draw
    for nny = 1:Ny
      patchData(nnx).x(:, nny) = nnx + [0, 0, 1, 1]';
      patchData(nnx).y(:, nny) = nny + [0, 1, 1, 0]';
      patchData(nnx).c(:, nny) = 0.2 * [1;1;1];

      if nIsHit(nny)
        if isDirectLight(nny)
          cp = cSurfHit(:, nny) * (0.3 + 0.7 * dotLight(nny));
        else
          cp = cSurfHit(:, nny) * (0.3 + 0.3 * dotLight(nny));
        end
        patchData(nnx).c(:, nny) = cp(:);
      end
    end
  end
  
%   % merge patch data
%   patchX = [patchData.x];
%   patchY = [patchData.y];
%   patchC = [patchData.c];
%   patchC = reshape(patchC', 1, Nx * Ny, 3);
% 
%   % initialize the axes
%   xp = [0, 0, Nx + 2, Nx + 2];
%   yp = [0, Ny + 2, Ny + 2, 0];
%   clf
%   patch(xp, yp, [0 0 0])
%   axis equal
%   axis tight
%   
%   % draw patches
%   patch(patchX, patchY, patchC, 'EdgeColor', 'none');
  
  % make an image and display it 
  img = uint16(zeros(Nx, 3, Ny));
  for n = 1:Nx
    img(n, :, :) = uint16(2^16 * fliplr(patchData(n).c));
  end
  img = permute(img, [3 1 2]);
  
  if nargout > 0
    varargout{1} = img;
  else
    image(img)
    axis equal
    axis tight
  end
end
  
    
