% drawMirror(par, vp, vz, vy, vx)
%   draw a mirror
%
% This is a right cylinder with the front surface centered at vp.
% The normal vector is vz, also defining the Z axis of the mirro's
% local basis.  The local X and Y vectors are vx and vy.

function drawMirror(dDiameter, dThick, vp, vz, vy, vx)

  Nf = 50;
  dth = 2 * pi / Nf;
  edgeParam = [0.5, 0.5, 0.8, 0.3, 2];
  backParam = [0.5, 1, 0.2, 0.2, 10];
  frontParam = [0.5, 1, 0.2, 0.2, 10];

  % draw cylinder in Nf pieces
  rad = dDiameter / 2;
  frnt = zeros(3, Nf + 1);
  back = zeros(3, Nf + 1);
  frnt(:, 1) = vp + vy * rad;
  back(:, 1) = frnt(:, 1) - vz * dThick;
  for n = 1:Nf
    frnt(:, n + 1) = vp + vecRot(vy * rad, vz, dth * n);
    back(:, n + 1) = frnt(:, n + 1) - vz * dThick;
    drawSurf([back(:, n:n+1), frnt(:, n+1:-1:n)], [0.7 0.5 1], edgeParam)
  end
  
  % draw front and back surfaces
  if frontParam(1) > 0
    drawSurf(frnt(:, 1:Nf), [0.7 0.5 1], frontParam);
  end
  
  if backParam(1) > 0
    drawSurf(back(:, 1:Nf), [0.7 0.5 1], backParam);
  end
  
  % draw basis lines
%   v0 = [0; 0; 0];
%   spos = [vp vp vp];
%   drawLine(spos - rad * [v0, vy, vx], spos + rad * [0.5 * vz, vy, vx], 'g');

end
