% [velNorm, vMB] = getFluxVelNorm(temp, mass, N)
%  temp = temperature in K
%  mass = molecular mass in atomic mass units
%  N = number of vales to generate (default = 1)
%
% Flux velocity is through a suface.  The normal velocity
% comes from a Rayleigh distribution, such that this represents
% the normal velocity distribution of particles which pass
% through the surface per unit time, assuming the particle
% velocity distribution is Maxwell-Boltzman.
% (see getMaxBoltzVel, getFluxVel)
%
% Rayleigh distribution:
% P(v) / dv = exp(-(v/vp)^2) * (2 * v / vp^2)


function [velNorm, vMB] = getFluxVelNorm(temp, mass, N)
  
  % constants
  ATOMIC_MASS = 1.66053886e-27;           % kg/au
  BOLTZMANN_CONST = 1.38066e-23;          % (kg m^2/s^2) / K

  % characteristic velocity
  vMB = sqrt(2 * (BOLTZMANN_CONST / ATOMIC_MASS) * (temp / mass));

  % normal velocity component with Rayleigh distribution
  %   P(v) / dv = exp(-(v/vp)^2) * (2 * v / vp^2)
  velNorm = vMB * sqrt(-log(1 - rand(N, 1)));

end
