% runGasBox(Ntotal, Ntrial, tEnd, cylPos, name)
%
%%%%%%%%%%%%%%%%%%%%%% Example
% % start the matlabpool
% matlabpool
%
% Ntotal = 1e6;
% Ntrial = 1e3;
% tEnd = 10;
% cylPos = [0.5, 0.5, 0.395; 0.5, 0.5, 0.5]';
% runGasBox(Ntotal, Ntrial, tEnd, cylPos, 'gasBoxClose')
%
% cylPos = [0.4, 0.4, 0.395; 0.6, 0.6, 0.5]';
% runGasBox(Ntotal, Ntrial, tEnd, cylPos, 'gasBoxSlide')

function rslt = runGasBox(p, Ntotal, name)

  % get parameter struct
  if isnumeric(p)
    p = getGasBoxParams(p);
  end
  
  % number of outer loops
  Ntrial = p.Ntrial;
  Nloop = ceil(Ntotal / Ntrial);
  
  % time stamp string
  now_str = datestr(now, 'yyyymmdd-HHMMSS');
  
  % save file name
  fileName = ['./Data/' name '_' now_str '.mat'];
  
  % a quick drawing
%  drawShapes(p.shapes, p.lightPos, p.cameraPos, ...
%    p.cameraDir, p.cameraFOV, 320, 240)
%  title(p.name, 'fontsize', 18)
%  xlabel(sprintf('%d sets of %d particles for %g seconds', ...
%    Nloop, Ntrial, p.tEnd))
%  drawnow

  %%%%%%%%%%%%%%%%%%%%%%
  % start running
  configInfo.batchTime = 2;
  configInfo.minBatchSize = 1;
  configInfo.enableWaitBar = false;
  configInfo.nameWaitBar = sprintf('Gas Box %d', Ntotal);
  
  % combine the results
  fprintf('==========> Starting %s at %s\n', name, now_str);
  tic
  rslt = pforfun(@gasBoxPar, p, 1:Nloop, configInfo);
  rslt = {rslt.data};
  rslt = gasBoxMergeRslt(rslt{1}, rslt(2:end));
  
  % done
  save(fileName, 'p', 'Ntotal', 'rslt')
  toc
  
end
